@@ -1,4 +1,4 @@
-//! cpython-json converts native Python objects (via cpython `PyObject`s) to serde_json `Value`s
+//! cpython-json converts native Python objects (via cpython `PyObject`s) to `serde_json::Value`s
 //! and back again.
 //!
 //! It was developed for [crowbar](https://crates.io/crates/crowbar), a shim for writing native
@@ -55,8 +55,7 @@
 //!     let sys = py.import("sys")?;
 //!     let py_hexversion = sys.get(py, "hexversion")?;
 //!     let hexversion = match to_json(py, &py_hexversion).map_err(|e| e.to_pyerr(py))? {
-//!         Value::U64(x) => x,
-//!         Value::I64(x) => x as u64,
+//!         Value::Number(x) => x.as_i64().expect("hexversion is not an int"),
 //!         _ => panic!("hexversion is not an int"),
 //!     };
 //!
@@ -65,31 +64,47 @@
 //! ```
 
 extern crate cpython;
+#[macro_use]
+extern crate quick_error;
 extern crate serde_json;
 
 use cpython::*;
 use serde_json::value::Value;
-use std::collections::BTreeMap;
 use std::convert::From;
 
-/// The `Error` enum returned by this crate.
-///
-/// Most of the time you will just want a `PyErr`, and `to_pyerr` will convert to one for you.
-#[derive(Debug)]
-pub enum JsonError {
-    /// A Python exception occurred.
-    PythonError(PyErr),
-    /// The PyObject passed could not be converted to a `serde_json::Value` object.
+quick_error! {
+    /// The `Error` enum returned by this crate.
     ///
-    /// The error tuple is the type name, then the `repr` of the object.
-    ///
-    /// This usually means that Python's `json` module wouldn't be able to serialize the object
-    /// either. If the Python `json` module works but `cpython-json` doesn't, please [file an
-    /// issue] (https://github.com/ilianaw/rust-cpython-json/issues) with your test case.
-    TypeError(String, PyResult<String>),
-    /// A `dict` key was not a string object, and so it couldn't be converted to an object. JSON
-    /// object keys must always be strings.
-    DictKeyNotString(PyObject),
+    /// Most of the time you will just want a `PyErr`, and `to_pyerr` will convert to one for you.
+    #[derive(Debug)]
+    pub enum JsonError {
+        /// A Python exception occurred.
+        PythonError(err: PyErr) {
+            from()
+        }
+        /// The PyObject passed could not be converted to a `serde_json::Value` because of a
+        /// `serde_json` error.
+        SerdeJsonError(err: serde_json::Error) {
+            from()
+        }
+        /// The PyObject passed could not be converted to a `serde_json::Value` object because we
+        /// didn't recognize it as a valid JSON-supported type.
+        ///
+        /// The error tuple is the type name, then the `repr` of the object.
+        ///
+        /// This usually means that Python's `json` module wouldn't be able to serialize the object
+        /// either. If the Python `json` module works but `cpython-json` doesn't, please [file an
+        /// issue] (https://github.com/ilianaw/rust-cpython-json/issues) with your test case.
+        TypeError(type_name: String, repr: PyResult<String>) {}
+        /// A `dict` key was not a string object, and so it couldn't be converted to an object. JSON
+        /// object keys must always be strings.
+        DictKeyNotString(obj: PyObject) {}
+        /// A number provided is not a valid JSON float (infinite and NaN values are not supported
+        /// in JSON).
+        InvalidFloat {}
+        /// The serde_json crate lied to us and a `Number` is neither u64, i64, or f64.
+        ImpossibleNumber {}
+    }
 }
 
 impl JsonError {
@@ -97,6 +112,13 @@ impl JsonError {
     pub fn to_pyerr(&self, py: Python) -> PyErr {
         match *self {
             JsonError::PythonError(ref err) => err.clone_ref(py),
+            JsonError::SerdeJsonError(_) => {
+                PyErr {
+                    ptype: cpython::exc::RuntimeError::type_object(py).into_object(),
+                    pvalue: Some(PyString::new(py, "serde_json error").into_object()),
+                    ptraceback: None,
+                }
+            }
             JsonError::TypeError(_, ref repr) => {
                 match *repr {
                     Ok(ref repr) => {
@@ -104,8 +126,8 @@ impl JsonError {
                             ptype: cpython::exc::TypeError::type_object(py).into_object(),
                             pvalue: Some(PyUnicode::new(py,
                                                         &format!("{} is not JSON serializable",
-                                                                 repr))
-                                .into_object()),
+                                                                repr))
+                                                 .into_object()),
                             ptraceback: None,
                         }
                     }
@@ -119,16 +141,24 @@ impl JsonError {
                     ptraceback: None,
                 }
             }
+            JsonError::InvalidFloat => {
+                PyErr {
+                    ptype: cpython::exc::ValueError::type_object(py).into_object(),
+                    pvalue: Some(PyString::new(py, "inf and nan are not supported in JSON").into_object()),
+                    ptraceback: None,
+                }
+            }
+            JsonError::ImpossibleNumber => {
+                PyErr {
+                    ptype: cpython::exc::ValueError::type_object(py).into_object(),
+                    pvalue: Some(PyString::new(py, "a value was somehow not an integer or float").into_object()),
+                    ptraceback: None,
+                }
+            }
         }
     }
 }
 
-impl From<PyErr> for JsonError {
-    fn from(err: PyErr) -> JsonError {
-        JsonError::PythonError(err)
-    }
-}
-
 /// Convert from a `cpython::PyObject` to a `serde_json::Value`.
 pub fn to_json(py: Python, obj: &PyObject) -> Result<Value, JsonError> {
     macro_rules! cast {
@@ -142,22 +172,22 @@ pub fn to_json(py: Python, obj: &PyObject) -> Result<Value, JsonError> {
     macro_rules! extract {
         ($t:ty) => {
             if let Ok(val) = obj.extract::<$t>(py) {
-                return Ok(serde_json::value::to_value(val));
+                return serde_json::value::to_value(val).map_err(JsonError::SerdeJsonError);
             }
         }
     }
 
     cast!(PyDict, |x: &PyDict| {
-        let mut map = BTreeMap::new();
+        let mut map = serde_json::Map::new();
         for (key_obj, value) in x.items(py) {
             let key = if key_obj == py.None() {
                 Ok("null".to_string())
             } else if let Ok(val) = key_obj.extract::<bool>(py) {
                 Ok(if val {
-                    "true".to_string()
-                } else {
-                    "false".to_string()
-                })
+                       "true".to_string()
+                   } else {
+                       "false".to_string()
+                   })
             } else if let Ok(val) = key_obj.str(py) {
                 Ok(val.to_string(py)?.into_owned())
             } else {
@@ -176,7 +206,11 @@ pub fn to_json(py: Python, obj: &PyObject) -> Result<Value, JsonError> {
     extract!(String);
     extract!(bool);
 
-    cast!(PyFloat, |x: &PyFloat| Ok(Value::F64(x.value(py))));
+    cast!(PyFloat,
+          |x: &PyFloat| match serde_json::Number::from_f64(x.value(py)) {
+              Some(n) => Ok(Value::Number(n)),
+              None => Err(JsonError::InvalidFloat),
+          });
 
     extract!(u64);
     extract!(i64);
@@ -186,7 +220,8 @@ pub fn to_json(py: Python, obj: &PyObject) -> Result<Value, JsonError> {
     }
 
     // At this point we can't cast it, set up the error object
-    let repr = obj.repr(py).and_then(|x| x.to_string(py).and_then(|y| Ok(y.into_owned())));
+    let repr = obj.repr(py)
+        .and_then(|x| x.to_string(py).and_then(|y| Ok(y.into_owned())));
     Err(JsonError::TypeError(obj.get_type(py).name(py).into_owned(), repr))
 }
 
@@ -199,9 +234,18 @@ pub fn from_json(py: Python, json: Value) -> Result<PyObject, JsonError> {
     }
 
     match json {
-        Value::I64(x) => obj!(x),
-        Value::U64(x) => obj!(x),
-        Value::F64(x) => obj!(x),
+        Value::Number(x) => {
+            if let Some(n) = x.as_u64() {
+                obj!(n)
+            } else if let Some(n) = x.as_i64() {
+                obj!(n)
+            } else if let Some(n) = x.as_f64() {
+                obj!(n)
+            } else {
+                // We should never get to this point
+                Err(JsonError::ImpossibleNumber)
+            }
+        }
         Value::String(x) => Ok(PyUnicode::new(py, &x).into_object()),
         Value::Bool(x) => obj!(x),
         Value::Array(vec) => {
@@ -265,7 +309,8 @@ mod tests {
             // test from_json
             if !line[2].contains("skip_from") {
                 let obj = py.eval(line[0], None, None).unwrap();
-                let eq = operator.call(py,
+                let eq = operator
+                    .call(py,
                           "__eq__",
                           PyTuple::new(py, &[from_json(py, json).unwrap(), obj]),
                           None)
@@ -285,7 +330,11 @@ mod tests {
 
         // datetime.datetime objects are not JSON serializable
         let datetime = py.import("datetime").unwrap();
-        let min = datetime.get(py, "datetime").unwrap().getattr(py, "min").unwrap();
+        let min = datetime
+            .get(py, "datetime")
+            .unwrap()
+            .getattr(py, "min")
+            .unwrap();
         let err = to_json(py, &min).unwrap_err().to_pyerr(py);
         assert_eq!(err.ptype, TypeError::type_object(py).into_object());
         assert_eq!(err.pvalue.unwrap().to_string(),
@@ -321,7 +370,7 @@ mod tests {
 
         let err = JsonError::TypeError("datetime.datetime".to_string(),
                                        Err(JsonError::DictKeyNotString(py.None()).to_pyerr(py)))
-            .to_pyerr(py);
+                .to_pyerr(py);
         assert_eq!(err.ptype, TypeError::type_object(py).into_object());
         assert_eq!(err.pvalue.unwrap().to_string(), "keys must be a string");
         assert_eq!(err.ptraceback, None);
